from zope import interface

class ITeaser(interface.Interface):
    """ Handler for teaser thumbing and captioning
    """
        
    def getTeaserURL(size="original"):
        """
        Returns the url to the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            teaser_<size>_(height|width)
        """
    
    def getTeaser(size="orginal"):
        """ 
        Returns the html tag of the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            teaser_<size>_(height|width)
        """
        
    def getSize(size):
        """
        Returns the width and height registered for the specific size
        """
    
    def getCaption():
        """
        Returns the caption for the image
        """


from zope import interface, component

from Products.CMFCore.utils import getToolByName

from raptus.article.core.interfaces import IArticle
from raptus.article.teaser.interfaces import ITeaser

class Teaser(object):
    """ Handler for teaser thumbing and captioning
    """
    interface.implements(ITeaser)
    component.adapts(IArticle)
    
    def __init__(self, context):
        self.context = context
        
    def getTeaserURL(self, size="original"):
        """
        Returns the url to the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            teaser_<size>_(height|width)
        """
        if not self.context.Schema()['image'].get(self.context):
            return None
        
        w, h = self.getSize(size)
        
        url = '%s/image' % self.context.absolute_url()
        if w or h:
            scales = component.getMultiAdapter((self.context, self.context.REQUEST), name='images')
            scale = scales.scale('image', width=(w or 100000), height=(h or 100000))
            if scale is not None:
                url = scale.url
        return url
        
    def getTeaser(self, size="original"):
        """ 
        Returns the html tag of the teaser image in the specific size
        
        The sizes are taken from the raptus_article properties sheet
        and are formed by the following name schema:
        
            teaser_<size>_(height|width)
        """
        url = self.getTeaserURL(size)
        if not url:
            return None
        
        return '<img src="%s" alt="%s" />' % (url, self.getCaption())
        
    def getSize(self, size):
        """
        Returns the width and height registered for the specific size
        """
        props = getToolByName(self.context, 'portal_properties').raptus_article
        return props.getProperty('teaser_%s_width' % size, 0), props.getProperty('teaser_%s_height' % size, 0)
    
    def getCaption(self, non_default = False):
        """
        Returns the caption for the image.
        If the flag non_default is set it's return a empty string if the caption are empty too
        """
        if (non_default):
            return self.context.Schema()['imageCaption'].get(self.context)
        else:
            return self.context.Schema()['imageCaption'].get(self.context) or self.context.Title()



from Acquisition import aq_inner
from zope import interface, component

from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase
from plone.memoize.instance import memoize

from raptus.article.core import RaptusArticleMessageFactory as _
from raptus.article.core import interfaces
from raptus.article.teaser.interfaces import ITeaser

class ITeaserLeft(interface.Interface):
    """ Marker interface for the teaser left viewlet
    """

class ComponentLeft(object):
    """ Component which shows the teaser image of an article on the left side
    """
    interface.implements(interfaces.IComponent)
    component.adapts(interfaces.IArticle)
    
    title = _(u'Image left')
    description = _(u'Image of the article on the left side.')
    image = '++resource++teaser_left.gif'
    interface = ITeaserLeft
    viewlet = 'raptus.article.teaser.left'
    
    def __init__(self, context):
        self.context = context

class ViewletLeft(ViewletBase):
    """ Viewlet showing the teaser image of an article on the left side
    """
    index = ViewPageTemplateFile('teaser.pt')
    css_class = "componentLeft teaser-left"
    type = "left"
    
    """ if the property are set the caption-text will display below the teaser-image
        as p.dicreet html tag
    """
    @property
    @memoize
    def show_caption(self):
        props = getToolByName(self.context, 'portal_properties').raptus_article
        return props.getProperty('teaser_%s_caption' % self.type, False)
    
    @property
    @memoize
    def caption(self):
        provider = ITeaser(self.context)
        return provider.getCaption()

    @property
    @memoize
    def caption_non_default(self):
        provider = ITeaser(self.context)
        return provider.getCaption(non_default = True)

    @property
    @memoize
    def image(self):
        provider = ITeaser(self.context)
        return provider.getTeaser(size=self.type)
    
    @property
    @memoize
    def url(self):
        provider = ITeaser(self.context)
        w, h = self.context.Schema()['image'].getSize(self.context)
        tw, th = provider.getSize(self.type)
        if (not tw or tw >= w) and (not th or th >= h):
            return None
        return provider.getTeaserURL(size="popup")

class ITeaserRight(interface.Interface):
    """ Marker interface for the teaser right viewlet
    """

class ComponentRight(object):
    """ Component which shows the teaser image of an article on the right side
    """
    interface.implements(interfaces.IComponent)
    component.adapts(interfaces.IArticle)
    
    title = _(u'Image right')
    description = _(u'Image of the article on the right side.')
    image = '++resource++teaser_right.gif'
    interface = ITeaserRight
    viewlet = 'raptus.article.teaser.right'
    
    def __init__(self, context):
        self.context = context

class ViewletRight(ViewletLeft):
    """ Viewlet showing the teaser image of an article on the right side
    """
    css_class = "componentRight teaser-right"
    type = "right"

class ITeaserFull(interface.Interface):
    """ Marker interface for the teaser full viewlet
    """

class ComponentFull(object):
    """ Component which shows the teaser image of an article over the whole width
    """
    interface.implements(interfaces.IComponent)
    component.adapts(interfaces.IArticle)
    
    title = _(u'Image')
    description = _(u'Image of the article over the whole width.')
    image = '++resource++teaser_full.gif'
    interface = ITeaserFull
    viewlet = 'raptus.article.teaser.full'
    
    def __init__(self, context):
        self.context = context

class ViewletFull(ViewletLeft):
    """ Viewlet showing the teaser image of an article over the whole width
    """
    css_class = "componentFull teaser-full"
    type = "full"



from zope.interface import implements
from zope.component import adapts

from archetypes.schemaextender.interfaces import ISchemaExtender
from archetypes.schemaextender.field import ExtensionField

try: # blob
    from plone.app.blob.field import ImageField
except:
    from Products.Archetypes.Field import ImageField

from Products.validation import V_REQUIRED
from Products.Archetypes import atapi
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes import ATCTMessageFactory as _at

from raptus.article.core import RaptusArticleMessageFactory as _
from raptus.article.core.interfaces import IArticle

class ImageField(ExtensionField, ImageField):
    """ ImageField
    """

class StringField(ExtensionField, atapi.StringField):
    """ StringField
    """
    
class ArticleExtender(object):
    """ Adds the image and caption fields to the article schema
    """
    implements(ISchemaExtender)
    adapts(IArticle)

    fields = [
        ImageField('image',
            required=False,
            languageIndependent=True,
            storage = atapi.AnnotationStorage(),
            swallowResizeExceptions = zconf.swallowImageResizeExceptions.enable,
            pil_quality = zconf.pil_config.quality,
            pil_resize_algo = zconf.pil_config.resize_algo,
            max_size = zconf.ATImage.max_image_dimension,
            sizes= {'large'   : (768, 768),
                    'preview' : (400, 400),
                    'mini'    : (200, 200),
                    'thumb'   : (128, 128),
                    'tile'    :  (64, 64),
                    'icon'    :  (32, 32),
                    'listing' :  (16, 16),
                   },
            validators = (('isNonEmptyFile', V_REQUIRED),
                          ('checkImageMaxSize', V_REQUIRED)),
            widget = atapi.ImageWidget(
                description = '',
                label= _at(u'label_image', default=u'Image'),
                show_content_type = False,
            )
        ),
        
        StringField('imageCaption',
            required = False,
            searchable = True,
            widget = atapi.StringWidget(
                description = '',
                label = _at(u'label_image_caption', default=u'Image Caption'),
                size = 40
            )
        ),
    ]

    def __init__(self, context):
         self.context = context
         
    def getFields(self):
        return self.fields



# 

from Products.ATContentTypes.content.image import ATImage
from Products.Archetypes.fieldproperty import ATFieldProperty

from raptus.article.core.content import article

def _tag(self, **kwargs):
    """Generate image tag using the api of the ImageField
    """
    return self.getField('image').tag(self, **kwargs)

def __bobo_traverse__(self, REQUEST, name):
    """Transparent access to image scales
    """
    if name.startswith('image'):
        field = self.getField('image')
        image = None
        if name == 'image':
            image = field.getScale(self)
        else:
            scalename = name[len('image_'):]
            if scalename in field.getAvailableSizes(self):
                image = field.getScale(self, scale=scalename)
        if image is not None and not isinstance(image, basestring):
            # image might be None or '' for empty images
            return image

    return super(article.Article, self).__bobo_traverse__(REQUEST, name)

article.Article.tag = _tag
article.Article.__bobo_traverse__ = __bobo_traverse__
article.Article.image = ATFieldProperty('image')